require 'pps_commons'

module WGU
  class Sentence
    include WGU::PPSCommons

    attr_reader :recipient
    attr_reader :message

    def initialize(recipient, msg)
      @recipient = recipient
      @message = (msg.kind_of?(WGU::Sentence) ? msg.log_entry : msg.to_s)
      @message.strip!
    end

    def add_need(need)
      with_clause =~ /#{need}/ ? @message : add_need!(need)
    end

    def add_need!(need)
      if with_clause.empty?
        @message << " with needs #{need}"
      else
        tmp = if with_clause =~ /needs/
          "needs #{need} and"
        else
          "needs #{need}"
        end

        new_with_clause = "#{tmp} #{with_clause}"
        @message.gsub!(/#{with_clause}/, new_with_clause).strip!
        @message
      end
    end

    def category
      request? ? :request : :response
    end

    def has_more_needs?
      !!(message.index(' needs '))
    end

    # Hostname or ipaddress from message
    def location_information
      if message.scan(/\s([a-z0-9\-]+\.wgu\.edu)\s/).count > 0
        message.gsub(/.+\s([a-z0-9\-]+\.wgu\.edu)\s.+/, '\1')
      elsif message.scan(/(\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b)/).count > 0
        message.gsub(/.+(\b\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\b).+/, '\1')
      elsif message.scan(/WGU_UT\\[a-z\-0-9]+\s/i).count > 0
        message.gsub(/.+WGU_UT\\([a-z\-0-9]+)\s.*/i, '\1')
      else
        message.gsub(/.+for\s\w+\son\s([\w\-]+)\swith.*/, '\1')
      end
    end

    def log_entry
      @log_entry || "#{message.to_s}\n"
    end

    def mark_completed!(need)
      matched_needs = needs.select { |nd| /\!?#{need}/ =~ nd }
      if matched_needs.count > 1
        "alert human #{message} has too many validations that match the name \"#{need}\" so it failed"
      else
        suffix = with_clause[/needs\s\!?#{Regexp.escape(matched_needs.first)}\s?/]
        new_suffix = suffix.gsub(/needs\s/, '')
        message.gsub!(/#{Regexp.escape(suffix)}/, new_suffix).strip!
        words = message.split(' ')
        prefix = words[0..words.index('for')].join(' ')
        message.gsub!(/#{Regexp.escape(prefix)}/, '').strip!
        message
      end
    end

    def mark_incompleted!(need)
      mark_completed!(need)
      message.gsub!(/(with)?(and)?\s!?#{need}/, '')
      add_need!("!#{need}")
    end

    def needs
      if has_more_needs?
        with_clause.split(' and ')
          .map { |need| need.gsub!(/(needs\s)/, '') }
      else
        []
      end
    end

    def next_tick
      if has_more_needs?
        next_need =
          needs.find { |need| need !~ /^!/ } ||
          (needs.find { |need| need =~ /^!/ }.gsub('!', '') rescue '')
        "validate #{next_need}"
      else
        [verb, simple_subject].join(' ')
      end
    end

    def new_request?
      !!(with_clause.empty? && needs.empty?)
    end

    def prepositions
      bare_patterns = [
        /\sunder\s/, /\sfor\s/, /\sover\s/, /\swithin\s/, /\saliased\s/, /\sat\s/,
        /\severy\s/
      ]
      if block_given?
        Regexp.union(bare_patterns.map { |pat| yield pat })
      else
        Regexp.union(bare_patterns)
      end
    end

    def request?
      !!(message =~ /^please/)
    end

    def simple_subject
      subject.gsub(
        /(.*?)#{prepositions { |prep| /#{prep.source + '.*'}/ }}/,
        '\1'
      )
    end

    def secondary_subject
      subject.gsub(/.+for(.*)$/, '\1').strip
    end

    def sentiment
      @sentiment || (
        (message =~ /#{success_patterns}$/) ? :success : :fail
      )
    end

    def subject(msg = message)
      words = msg.split
      if request?
        lower_bound = 2
        upper_bound = (words.reverse.index('with') || words.count)
        words[lower_bound...upper_bound].join(' ')
      else
        words[0..-2].join(' ')
      end
    end

    def success_patterns
      Regexp.union([
        /(connected)/, /(success)/, /(succeeded)/, /(successfull)/,
        /(working)/, /(works)/, /(linked)/, /(done)/, /(skipped)/
      ])
    end

    def ticked!(status)
      words = message.split(' ')
      words.shift # get rid of "please"
      words.first.gsub!(/(\w+)e$/, "#{'\1'}ion") # change from imperative to declarative
      words.insert(1, 'for') # finish changing to declarative sentence
      words.push(status) # add the status of the task that ran to the end
      @message = words.join(' ')
      message
    end

    def verb
      words = message.split(' ')
      request? ? words[1] : words.last
    end

    def with_clause
      return '' unless message =~ /with/
      msg = message.gsub(/.+with\s(.+)$/, '\1')
      if request?
        msg
      else
        # pop the response status off, when dealing with a response
        tmp = msg.split(' ')
        tmp.pop
        tmp.join(' ').strip
      end
    end
  end
end
